@@ -1,7 +1,7 @@
 /* acconfig.h: Descriptions of macros produced by autoconf
    Copyright (c) 2000,2002 Philip Kendall
 
-   $Id: acconfig.h,v 1.14 2002/01/09 17:00:20 pak21 Exp $
+   $Id: acconfig.h,v 1.21 2003/06/23 16:17:18 pak21 Exp $
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -26,12 +26,18 @@
 
 @TOP@
 
+/* Defined if we're in debugging mode */
+#undef DEBUG_MODE
+
 /* Defined if we've got enough memory to compile z80_ops.c */
 #undef HAVE_ENOUGH_MEMORY
 
 /* Defined if we've got glib */
 #undef HAVE_LIB_GLIB
 
+/* Defined if we've got libxml2 */
+#undef HAVE_LIB_XML2
+
 /* Defined if Issue 2 emulation requested */
 #undef ISSUE2
 
@@ -41,11 +47,26 @@
 /* Defined if framebuffer UI in use */
 #undef UI_FB
 
-/* Defined if GTK+ UI in use */
+/* Defined if GGI UI in use */
+#undef UI_GGI
+
+/* Defined if GTK+ UI (either 1.2 or 2.x) is in use */
 #undef UI_GTK
 
+/* Defined if GTK+ 2.0 is in use */
+#undef UI_GTK2
+
+/* Defined if the SDL UI in use */
+#undef UI_SDL
+
 /* Defined if svgalib UI in use */
 #undef UI_SVGA
 
 /* Defined if Xlib UI in use */
 #undef UI_X
+
+/* Defined if we're going to be using the installed libpng */
+#undef USE_LIBPNG
+
+/* Defined if we're using a widget-based UI */
+#undef USE_WIDGET
