@@ -1,7 +1,7 @@
 /* event.c: Routines needed for dealing with the event list
-   Copyright (c) 2000,2002 Philip Kendall
+   Copyright (c) 2000-2003 Philip Kendall
 
-   $Id: event.c,v 1.25 2002/03/21 21:53:36 pak21 Exp $
+   $Id: event.c,v 1.37 2003/07/11 11:03:25 pak21 Exp $
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -29,28 +29,31 @@
 #include <stdio.h>
 #include <stdlib.h>
 
-#ifdef HAVE_LIB_GLIB		/* If we're using glib */
-#include <glib.h>
-#else				/* #ifdef HAVE_LIB_GLIB */
-#include "myglib/myglib.h"	/* if not, use the local replacement */
-#endif
+#include <libspectrum.h>
 
 #include "display.h"
 #include "event.h"
+#include "fuse.h"
 #include "machine.h"
+#include "psg.h"
 #include "rzx.h"
 #include "tape.h"
+#include "trdos.h"
 #include "ui/ui.h"
 #include "spectrum.h"
+#include "z80/z80.h"
 
 /* A large value to mean `no events due' */
-const DWORD event_no_events = 0xffffffff;
+const libspectrum_dword event_no_events = 0xffffffff;
+
+/* When will the next event happen? */
+libspectrum_dword event_next_event;
 
 /* The actual list of events */
 static GSList* event_list;
 
 /* Comparision function so events stay in t-state order */
-gint event_add_cmp(gconstpointer a, gconstpointer b);
+static gint event_add_cmp( gconstpointer a, gconstpointer b );
 
 /* User function for event_interrupt(...) */
 void event_reduce_tstates(gpointer data,gpointer user_data);
@@ -73,7 +76,8 @@ int event_init(void)
 }
 
 /* Add an event at the correct place in the event list */
-int event_add(DWORD event_time, int type)
+int
+event_add( libspectrum_dword event_time, int type )
 {
   event_t *ptr;
 
@@ -91,18 +95,10 @@ int event_add(DWORD event_time, int type)
 }
 
 /* Comparision function so events stay in t-state order */
-gint event_add_cmp(gconstpointer a, gconstpointer b)
+static gint
+event_add_cmp( gconstpointer a, gconstpointer b )
 {
-  const event_t *ptr1=(const event_t *)a;
-  const event_t *ptr2=(const event_t *)b;
-
-  if(ptr1->tstates < ptr2->tstates) {
-    return -1;
-  } else if(ptr1->tstates > ptr2->tstates) {
-    return 1;
-  } else {
-    return 0;
-  }
+  return ((const event_t*)a)->tstates - ((const event_t*)b)->tstates;
 }
 
 /* Do all events which have passed */
@@ -130,9 +126,10 @@ int event_do_events(void)
 
     case EVENT_TYPE_INTERRUPT:
       if( rzx_playback ) event_force_events();
+      rzx_frame();
+      psg_frame();
       spectrum_interrupt();
       ui_event();
-      rzx_frame();
       break;
 
     case EVENT_TYPE_LINE:
@@ -143,6 +140,18 @@ int event_do_events(void)
       tape_next_edge( ptr->tstates );
       break;
 
+    case EVENT_TYPE_NMI:
+      z80_nmi();
+      break;
+
+    case EVENT_TYPE_TRDOS_CMD_DONE:
+      trdos_event_cmd_done( ptr->tstates );
+      break;
+
+    case EVENT_TYPE_TRDOS_INDEX:
+      trdos_event_index( ptr->tstates );
+      break;
+
     default:
       ui_error( UI_ERROR_ERROR, "unknown event type %d", ptr->type );
       break;
@@ -154,7 +163,8 @@ int event_do_events(void)
 }
 
 /* Called on interrupt to reduce T-state count of all entries */
-int event_interrupt( DWORD tstates_per_frame )
+int
+event_interrupt( libspectrum_dword tstates_per_frame )
 {
   g_slist_foreach(event_list, event_reduce_tstates, &tstates_per_frame );
 
@@ -171,7 +181,7 @@ int event_interrupt( DWORD tstates_per_frame )
 void event_reduce_tstates(gpointer data,gpointer user_data)
 {
   event_t *ptr=(event_t*)data;
-  DWORD *tstates_per_frame = (DWORD*)user_data;
+  libspectrum_dword *tstates_per_frame = (libspectrum_dword*)user_data;
 
   ptr->tstates -= (*tstates_per_frame) ;
 }
@@ -204,7 +214,8 @@ int event_reset(void)
 }
 
 /* Free the memory used by a specific entry */
-void event_free_entry(gpointer data, gpointer user_data)
+void
+event_free_entry( gpointer data, gpointer user_data GCC_UNUSED )
 {
   event_t *ptr=(event_t*)data;
   free(ptr);
@@ -215,7 +226,7 @@ void event_free_entry(gpointer data, gpointer user_data)
    effect */
 static int event_force_events( void )
 {
-  while( event_next_event < machine_current->timings.cycles_per_frame ) {
+  while( event_next_event < machine_current->timings.tstates_per_frame ) {
 
     /* Jump along to the next event */
     tstates = event_next_event;
@@ -226,7 +237,7 @@ static int event_force_events( void )
   }
 
   /* Finally, jump to the interrupt time */
-  tstates = machine_current->timings.cycles_per_frame;
+  tstates = machine_current->timings.tstates_per_frame;
 
   return 0;
 }
