@@ -1,7 +1,7 @@
-/* event.c: Routines needed for dealing with the event list
-   Copyright (c) 2000 Philip Kendall
+/* event.h: Routines needed for dealing with the event list
+   Copyright (c) 2000-2003 Philip Kendall
 
-   $Id: event.h,v 1.9 2002/02/09 00:33:38 pak21 Exp $
+   $Id: event.h,v 1.13 2003/07/11 11:03:25 pak21 Exp $
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -27,13 +27,11 @@
 #ifndef FUSE_EVENT_H
 #define FUSE_EVENT_H
 
-#ifndef FUSE_TYPES_H
-#include "types.h"
-#endif			/* #ifndef FUSE_TYPES_H */
+#include <libspectrum.h>
 
 /* Information about an event */
 typedef struct event_t {
-  DWORD tstates;
+  libspectrum_dword tstates;
   int type;
 } event_t;
 
@@ -43,25 +41,28 @@ enum event_types {
   EVENT_TYPE_INTERRUPT,
   EVENT_TYPE_LINE,
   EVENT_TYPE_EDGE,
+  EVENT_TYPE_NMI,
+  EVENT_TYPE_TRDOS_CMD_DONE,
+  EVENT_TYPE_TRDOS_INDEX,
 };
 
 /* A large value to mean `no events due' */
-extern const DWORD event_no_events;
+extern const libspectrum_dword event_no_events;
 
 /* When will the next event happen? */
-DWORD event_next_event;
+extern libspectrum_dword event_next_event;
 
 /* Set up the event list */
 int event_init(void);
 
 /* Add an event at the correct place in the event list */
-int event_add(DWORD event_time, int type);
+int event_add( libspectrum_dword event_time, int type );
 
 /* Do all events which have passed */
 int event_do_events(void);
 
 /* Called on interrupt to reduce T-state count of all entries */
-int event_interrupt( DWORD tstates_per_frame );
+int event_interrupt( libspectrum_dword tstates_per_frame );
 
 /* Remove all events of a specific type from the stack */
 int event_remove_type( int type );
