@@ -1,7 +1,7 @@
 /* fuse.h: Variables exported from the main file
-   Copyright (c) 2000 Philip Kendall
+   Copyright (c) 2000-2003 Philip Kendall
 
-   $Id: fuse.h,v 1.10 2002/03/21 11:58:05 pak21 Exp $
+   $Id: fuse.h,v 1.14 2003/06/20 20:22:19 pak21 Exp $
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -27,6 +27,8 @@
 #ifndef FUSE_FUSE_H
 #define FUSE_FUSE_H
 
+#include <libspectrum.h>
+
 extern char* fuse_progname;		/* argv[0] */
 
 extern int fuse_exiting;		/* Shall we exit now? */
@@ -39,4 +41,7 @@ int fuse_abort( void );			/* Emergency shutdown */
 
 extern int fuse_sound_in_use;		/* Are we trying to produce sound? */
 
+extern libspectrum_creator *fuse_creator; /* Creator information for file
+					     formats which support this */
+
 #endif			/* #ifndef FUSE_FUSE_H */
