@@ -1,7 +1,7 @@
 /* joystick.h: Joystick emulation support
-   Copyright (c) 2001 Russell Marks, Philip Kendall
+   Copyright (c) 2001-2003 Russell Marks, Philip Kendall
 
-   $Id: joystick.h,v 1.1 2001/12/15 14:22:53 pak21 Exp $
+   $Id: joystick.h,v 1.3 2003/07/11 11:03:25 pak21 Exp $
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -27,11 +27,10 @@
 #ifndef FUSE_JOYSTICK_H
 #define FUSE_JOYSTICK_H
 
-#ifndef FUSE_TYPES_H
-#include "types.h"
-#endif			/* #ifndef FUSE_TYPES_H */
+#include <libspectrum.h>
 
-BYTE joystick_kempston_read(WORD port);
-void joystick_kempston_write(WORD port, BYTE b);
+libspectrum_byte joystick_kempston_read( libspectrum_word port );
+void joystick_kempston_write( libspectrum_word port, libspectrum_byte b );
+libspectrum_byte joystick_timex_read( libspectrum_word port, int which );
 
 #endif			/* #ifndef FUSE_JOYSTICK_H */
