@@ -1,7 +1,7 @@
 /* keysyms.h: keysym to Spectrum key mappings for both Xlib and GDK
-   Copyright (c) 2000 Philip Kendall
+   Copyright (c) 2000,2002 Philip Kendall
 
-   $Id: keysyms.h,v 1.6 2001/08/13 21:49:40 pak21 Exp $
+   $Id: keysyms.h,v 1.7 2002/12/11 15:29:23 pak21 Exp $
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -36,6 +36,6 @@ typedef struct keysyms_key_info {
   keyboard_key_name key1,key2;
 } keysyms_key_info;
 
-keysyms_key_info* keysyms_get_data(unsigned keysym);
+const keysyms_key_info* keysyms_get_data( unsigned keysym );
 
 #endif			/* #ifndef FUSE_KEYSYMS_H */
