@@ -1,5 +1,5 @@
 /* sound.c: Sound support
-   Copyright (c) 2000-2002 Russell Marks, Matan Ziv-Av, Philip Kendall
+   Copyright (c) 2000-2003 Russell Marks, Matan Ziv-Av, Philip Kendall
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -44,9 +44,12 @@
 #include <sys/ioctl.h>
 #include <fcntl.h>
 
+#include <libspectrum.h>
+
 #include "fuse.h"
 #include "machine.h"
 #include "osssound.h"
+#include "sdlsound.h"
 #include "settings.h"
 #include "sound.h"
 #include "spectrum.h"
@@ -60,15 +63,13 @@ int sound_enabled_ever=0;	/* if it's *ever* been in use; see
                                    sound_ay_write() and sound_ay_reset() */
 int sound_freq=32000;
 int sound_stereo=0;		/* true for stereo *output sample* (only) */
-int sound_stereo_beeper=0;	/* beeper pseudo-stereo */
 int sound_stereo_ay_abc=0;	/* (AY stereo) true for ABC stereo, else ACB */
 int sound_stereo_ay_narrow=0;	/* (AY stereo) true for narrow AY st. sep. */
 
 static int sound_stereo_ay=0;	/* local copy of settings_current.stereo_ay */
+static int sound_stereo_beeper=0;   /* and settings_current.stereo_beeper */
 
 
-#define AY_CLOCK		1773400
-
 /* assume all three tone channels together match the beeper volume (ish).
  * Must be <=127 for all channels; 50+2+(24*3) = 124.
  */
@@ -94,11 +95,6 @@ static unsigned char *sound_buf,*tape_buf;
 static int sound_oldpos[2],sound_fillpos[2];
 static int sound_oldval[2],sound_oldval_orig[2];
 
-/* timer used for fadeout after beeper-toggle;
- * fixed-point with low 24 bits as fractional part.
- */
-static unsigned int beeper_tick[2],beeper_tick_incr;
-
 /* foo_subcycles are fixed-point with low 16 bits as fractional part.
  * The other bits count as the chip does.
  */
@@ -111,11 +107,11 @@ static unsigned int ay_tone_period[3],ay_noise_period,ay_env_period;
 static int beeper_last_subpos[2]={0,0};
 
 /* Local copy of the AY registers */
-static BYTE sound_ay_registers[16];
+static libspectrum_byte sound_ay_registers[16];
 
 struct ay_change_tag
   {
-  DWORD tstates;
+  libspectrum_dword tstates;
   unsigned short ofs;
   unsigned char reg,val;
   };
@@ -159,7 +155,7 @@ ay_tone_subcycles=ay_env_subcycles=0;
 for(f=0;f<3;f++)
   ay_tone_tick[f]=ay_tone_high[f]=0,ay_tone_period[f]=1;
 
-ay_tick_incr=(int)(65536.*AY_CLOCK/sound_freq);
+ay_tick_incr=(int)(65536.*libspectrum_timings_ay_speed(machine_current->machine)/sound_freq);
 
 ay_change_count=0;
 }
@@ -171,7 +167,7 @@ static int first_init=1;
 int f,ret;
 
 /* if we don't have any sound I/O code compiled in, don't do sound */
-#if !defined(HAVE_SYS_SOUNDCARD_H) && !defined(HAVE_SYS_AUDIOIO_H)
+#if !defined(HAVE_SYS_SOUNDCARD_H) && !defined(HAVE_SYS_AUDIOIO_H) && !defined(UI_SDL)
 return;
 #endif
 
@@ -184,15 +180,17 @@ if(sound_enabled)
   }
 
 sound_stereo_ay=settings_current.stereo_ay;
+sound_stereo_beeper=settings_current.stereo_beeper;
 
 /* only try for stereo if we need it */
 if(sound_stereo_ay || sound_stereo_beeper)
   sound_stereo=1;
 
-#if defined(HAVE_SYS_SOUNDCARD_H)
+#if defined(UI_SDL)
+ret=sdlsound_init(device,&sound_freq,&sound_stereo);
+#elif defined(HAVE_SYS_SOUNDCARD_H)
 ret=osssound_init(device,&sound_freq,&sound_stereo);
-#endif
-#if defined(HAVE_SYS_AUDIOIO_H)
+#elif defined(HAVE_SYS_AUDIOIO_H)
 ret=sunsound_init(device,&sound_freq,&sound_stereo);
 #endif
 
@@ -200,9 +198,9 @@ if(ret)
   return;
 
 /* important to override these settings if not using stereo
- * (it would probably be confusing to mess with the AY stereo
- * setting in settings_current though, which is why we make a copy
- * rather than using the real one).
+ * (it would probably be confusing to mess with the stereo
+ * settings in settings_current though, which is why we make copies
+ * rather than using the real ones).
  */
 if(!sound_stereo)
   {
@@ -243,14 +241,8 @@ if(first_init)
   {
   first_init=0;
 
-  beeper_tick_incr=(1<<24)/sound_freq;
   for(f=0;f<2;f++)
-    {
     sound_oldval[f]=sound_oldval_orig[f]=128;
-    beeper_tick[f]=0;
-    }
-  
-  sound_ay_init();
   }
 
 if(sound_stereo_beeper)
@@ -287,10 +279,11 @@ if(sound_enabled)
   {
   if(sound_buf)
     free(sound_buf);
-#if defined(HAVE_SYS_SOUNDCARD_H)
+#if defined(UI_SDL)
+  sdlsound_end();
+#elif defined(HAVE_SYS_SOUNDCARD_H)
   osssound_end();
-#endif
-#if defined(HAVE_SYS_AUDIOIO_H)
+#elif defined(HAVE_SYS_AUDIOIO_H)
   sunsound_end();
 #endif
   sound_enabled=0;
@@ -409,7 +402,7 @@ if(!machine_current->ay.present) return;
 /* convert change times to sample offsets */
 for(f=0;f<ay_change_count;f++)
   ay_change[f].ofs=(ay_change[f].tstates*sound_freq)/
-                   machine_current->timings.hz;
+                   machine_current->timings.processor_speed;
 
 for(f=0,ptr=sound_buf;f<sound_framesiz;f++)
   {
@@ -628,7 +621,7 @@ for(f=0,ptr=sound_buf;f<sound_framesiz;f++)
 /* don't make the change immediately; record it for later,
  * to be made by sound_frame() (via sound_ay_overlay()).
  */
-void sound_ay_write(int reg,int val,DWORD now)
+void sound_ay_write(int reg,int val,libspectrum_dword now)
 {
 /* have to allow it across pauses for snap-loading to work,
  * so see if sound has *ever* been enabled.
@@ -655,6 +648,9 @@ int f;
 /* as above... */
 if(!sound_enabled_ever) return;
 
+/* recalculate timings based on new machines ay clock */
+sound_ay_init();
+
 ay_change_count=0;
 for(f=0;f<16;f++)
   sound_ay_write(f,0,0);
@@ -664,34 +660,6 @@ ay_tone_subcycles=ay_env_subcycles=0;
 }
 
 
-/* it should go without saying that the beeper was hardly capable of
- * generating perfect square waves. :-) What seems to have happened is
- * that after the `click' in the intended direction away from the rest
- * (zero) position, it faded out gradually over about 1/50th of a second
- * back down to zero - the bulk of the fade being over after about
- * a 1/500th.
- *
- * The true behaviour is awkward to model accurately, but a compromise
- * emulation (doing a linear fadeout over 1/150th) seems to work quite
- * well and IMHO sounds a little more like a real speccy than most
- * emulations. It also has the considerable advantage of having a zero
- * rest position, which I'm a lot happier with. :-)
- */
-
-#define BEEPER_FADEOUT	(((1<<24)/150)/ampl)
-
-#define BEEPER_OLDVAL_ADJUST(is_tape) \
-  beeper_tick[is_tape]+=beeper_tick_incr;	\
-  if(beeper_tick[is_tape]>=BEEPER_FADEOUT)	\
-    {						\
-    beeper_tick[is_tape]-=BEEPER_FADEOUT;	\
-    if(sound_oldval[is_tape]>128)		\
-      sound_oldval[is_tape]--;			\
-    else					\
-      if(sound_oldval[is_tape]<128)		\
-        sound_oldval[is_tape]++;		\
-    }
-
 /* write stereo or mono beeper sample, and incr ptr */
 #define SOUND_WRITE_BUF_BEEPER(ptr,val) \
   do						\
@@ -733,10 +701,7 @@ ptr=sound_buf+(sound_stereo?sound_fillpos[0]*2:sound_fillpos[0]);
 for(bchan=0;bchan<2;bchan++)
   {
   for(f=sound_fillpos[bchan];f<sound_framesiz;f++)
-    {
-    BEEPER_OLDVAL_ADJUST(bchan);
     SOUND_WRITE_BUF(bchan,ptr,sound_oldval[bchan]);
-    }
 
   ptr=tape_buf+sound_fillpos[1];
   ampl=AMPL_TAPE;
@@ -755,10 +720,11 @@ for(f=0;f<sound_framesiz;f++,tptr++)
 /* overlay AY sound */
 sound_ay_overlay();
 
-#if defined(HAVE_SYS_SOUNDCARD_H)
+#if defined(UI_SDL)
+sdlsound_frame(sound_buf,sound_framesiz*sound_channels);
+#elif defined(HAVE_SYS_SOUNDCARD_H)
 osssound_frame(sound_buf,sound_framesiz*sound_channels);
-#endif
-#if defined(HAVE_SYS_AUDIOIO_H)
+#elif defined(HAVE_SYS_AUDIOIO_H)
 sunsound_frame(sound_buf,sound_framesiz*sound_channels);
 #endif
 
@@ -791,10 +757,10 @@ if(val==sound_oldval_orig[bchan]) return;
 /* XXX a lookup table might help here, but would need to regenerate it
  * whenever cycles_per_frame were changed (i.e. when machine type changed).
  */
-newpos=(tstates*sound_framesiz)/machine_current->timings.cycles_per_frame;
+newpos=(tstates*sound_framesiz)/machine_current->timings.tstates_per_frame;
 /* the >>1s are to avoid overflow when int is 32-bit */
 subpos=((tstates>>1)*sound_framesiz*vol)/
-       (machine_current->timings.cycles_per_frame>>1)-vol*newpos;
+       (machine_current->timings.tstates_per_frame>>1)-vol*newpos;
 
 /* if we already wrote here, adjust the level.
  */
@@ -816,8 +782,6 @@ subval=128+ampl-beeper_last_subpos[bchan];
 
 if(newpos>=0)
   {
-  int oldval=sound_oldval[bchan];
-  
   /* fill gap from previous position */
   if(is_tape)
     ptr=tape_buf+sound_fillpos[1];
@@ -825,10 +789,7 @@ if(newpos>=0)
     ptr=sound_buf+(sound_stereo?sound_fillpos[0]*2:sound_fillpos[0]);
   
   for(f=sound_fillpos[bchan];f<newpos && f<sound_framesiz;f++)
-    {
-    BEEPER_OLDVAL_ADJUST(bchan);
-    SOUND_WRITE_BUF(bchan,ptr,oldval);
-    }
+    SOUND_WRITE_BUF(bchan,ptr,sound_oldval[bchan]);
 
   if(newpos<sound_framesiz)
     {
@@ -838,13 +799,6 @@ if(newpos>=0)
     else
       ptr=sound_buf+(sound_stereo?newpos*2:newpos);
 
-    /* limit subval in case of faded beeper level,
-     * to avoid slight spikes on ordinary tones.
-     */
-    if((oldval<128 && subval<oldval) ||
-       (oldval>=128 && subval>oldval))
-      subval=oldval;
-
     /* write subsample value */
     SOUND_WRITE_BUF(bchan,ptr,subval);
     }
