@@ -1,7 +1,7 @@
 /* spec48.c: Spectrum 48K specific routines
-   Copyright (c) 1999-2002 Philip Kendall
+   Copyright (c) 1999-2003 Philip Kendall
 
-   $Id: spec48.c,v 1.30 2002/03/21 21:53:37 pak21 Exp $
+   $Id: spec48.c,v 1.46 2003/07/11 11:03:25 pak21 Exp $
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -28,76 +28,40 @@
 
 #include <stdio.h>
 
+#include <libspectrum.h>
+
 #include "display.h"
 #include "fuse.h"
 #include "joystick.h"
-#include "keyboard.h"
 #include "machine.h"
 #include "printer.h"
-#include "snapshot.h"
-#include "sound.h"
+#include "settings.h"
 #include "spec48.h"
 #include "spectrum.h"
-#include "ui/ui.h"
-#include "z80/z80.h"
 
-static DWORD spec48_contend_delay( void );
+static libspectrum_dword spec48_contend_delay( void );
 
 spectrum_port_info spec48_peripherals[] = {
   { 0x0001, 0x0000, spectrum_ula_read, spectrum_ula_write },
   { 0x0004, 0x0000, printer_zxp_read, printer_zxp_write },
-  { 0x00e0, 0x0000, joystick_kempston_read, joystick_kempston_write },
+  { 0x00e0, 0x0000, joystick_kempston_read, spectrum_port_nowrite },
   { 0, 0, NULL, NULL } /* End marker. DO NOT REMOVE */
 };
 
-static BYTE spec48_unattached_port( void )
+static libspectrum_byte
+spec48_unattached_port( void )
 {
   return spectrum_unattached_port( 1 );
 }
 
-BYTE spec48_readbyte(WORD address)
-{
-  WORD bank;
-
-  if(address<0x4000) return ROM[0][address];
-  bank=address/0x4000; address-=(bank*0x4000);
-  switch(bank) {
-    case 1: return RAM[5][address]; break;
-    case 2: return RAM[2][address]; break;
-    case 3: return RAM[0][address]; break;
-    default:
-      ui_error( UI_ERROR_ERROR, "access to impossible bank %d at %s:%d",
-		bank, __FILE__, __LINE__ );
-      fuse_abort();
-  }
-  return 0; /* Keep gcc happy */
-}
-
-BYTE spec48_read_screen_memory(WORD offset)
+libspectrum_byte
+spec48_read_screen_memory( libspectrum_word offset )
 {
   return RAM[5][offset];
 }
 
-void spec48_writebyte(WORD address, BYTE b)
-{
-  if(address>=0x4000) {		/* 0x4000 = 1st byte of RAM */
-    WORD bank=address/0x4000,offset=address-(bank*0x4000);
-    switch(bank) {
-    case 1: RAM[5][offset]=b; break;
-    case 2: RAM[2][offset]=b; break;
-    case 3: RAM[0][offset]=b; break;
-    default:
-      ui_error( UI_ERROR_ERROR, "access to impossible bank %d at %s:%d",
-		bank, __FILE__, __LINE__ );
-      fuse_abort();
-    }
-    if(address<0x5b00) {	/* 0x4000 - 0x5aff = display file */
-      display_dirty( address );	/* Replot necessary pixels */
-    }
-  }
-}
-
-DWORD spec48_contend_memory( WORD address )
+libspectrum_dword
+spec48_contend_memory( libspectrum_word address )
 {
   /* Contention occurs only in the lowest 16Kb of RAM */
   if( address < 0x4000 || address > 0x7fff ) return 0;
@@ -105,7 +69,8 @@ DWORD spec48_contend_memory( WORD address )
   return spec48_contend_delay();
 }
 
-DWORD spec48_contend_port( WORD port )
+libspectrum_dword
+spec48_contend_port( libspectrum_word port )
 {
   /* Contention occurs only for even-numbered ports */
   if( ( port & 0x01 ) == 0 ) return spec48_contend_delay();
@@ -113,9 +78,10 @@ DWORD spec48_contend_port( WORD port )
   return 0;
 }
 
-static DWORD spec48_contend_delay( void )
+static libspectrum_dword
+spec48_contend_delay( void )
 {
-  WORD tstates_through_line;
+  libspectrum_word tstates_through_line;
   
   /* No contention in the upper border */
   if( tstates < machine_current->line_times[ DISPLAY_BORDER_HEIGHT ] )
@@ -128,16 +94,16 @@ static DWORD spec48_contend_delay( void )
 
   /* Work out where we are in this line */
   tstates_through_line =
-    ( tstates + machine_current->timings.left_border_cycles ) %
-    machine_current->timings.cycles_per_line;
+    ( tstates + machine_current->timings.left_border ) %
+    machine_current->timings.tstates_per_line;
 
   /* No contention if we're in the left border */
-  if( tstates_through_line < machine_current->timings.left_border_cycles - 1 ) 
+  if( tstates_through_line < machine_current->timings.left_border - 1 ) 
     return 0;
 
   /* Or the right border or retrace */
-  if( tstates_through_line >= machine_current->timings.left_border_cycles +
-                              machine_current->timings.screen_cycles - 1 )
+  if( tstates_through_line >= machine_current->timings.left_border +
+                              machine_current->timings.horizontal_screen - 1 )
     return 0;
 
   /* We now know the ULA is reading the screen, so put in the appropriate
@@ -156,44 +122,50 @@ static DWORD spec48_contend_delay( void )
   return 0;	/* Shut gcc up */
 }
 
-int spec48_init( machine_info *machine )
+int spec48_init( fuse_machine_info *machine )
 {
   int error;
 
-  machine->machine = SPECTRUM_MACHINE_48;
-  machine->description = "Spectrum 48K";
+  machine->machine = LIBSPECTRUM_MACHINE_48;
   machine->id = "48";
 
   machine->reset = spec48_reset;
 
-  machine_set_timings( machine, 3.5e6, 24, 128, 24, 48, 312, 8936 );
+  error = machine_set_timings( machine ); if( error ) return error;
 
-  machine->ram.read_memory    = spec48_readbyte;
-  machine->ram.read_screen    = spec48_read_screen_memory;
-  machine->ram.write_memory   = spec48_writebyte;
-  machine->ram.contend_memory = spec48_contend_memory;
-  machine->ram.contend_port   = spec48_contend_port;
+  machine->timex = 0;
+  machine->ram.read_memory           = spec48_readbyte;
+  machine->ram.read_memory_internal  = spec48_readbyte_internal;
+  machine->ram.read_screen           = spec48_read_screen_memory;
+  machine->ram.write_memory	     = spec48_writebyte;
+  machine->ram.write_memory_internal = spec48_writebyte_internal;
+  machine->ram.contend_memory        = spec48_contend_memory;
+  machine->ram.contend_port          = spec48_contend_port;
+  machine->ram.current_screen = 5;
 
   error = machine_allocate_roms( machine, 1 );
   if( error ) return error;
-  error = machine_read_rom( machine, 0, "48.rom" );
-  if( error ) return error;
+  machine->rom_length[0] = 0x4000;
 
   machine->peripherals = spec48_peripherals;
   machine->unattached_port = spec48_unattached_port;
 
   machine->ay.present = 0;
 
+  machine->shutdown = NULL;
+
   return 0;
 
 }
 
-int spec48_reset(void)
+int
+spec48_reset( void )
 {
-  z80_reset();
-  sound_ay_reset();	/* should happen for *all* resets */
-  snapshot_flush_slt();
-  printer_zxp_reset();
+  int error;
+
+  error = machine_load_rom( &ROM[0], settings_current.rom_48,
+			    machine_current->rom_length[0] );
+  if( error ) return error;
 
   return 0;
 }
