@@ -1,7 +1,7 @@
 /* spec48.h: Spectrum 48K specific routines
-   Copyright (c) 1999-2002 Philip Kendall
+   Copyright (c) 1999-2003 Philip Kendall
 
-   $Id: spec48.h,v 1.16 2002/01/20 16:33:58 pak21 Exp $
+   $Id: spec48.h,v 1.21 2003/07/11 11:03:25 pak21 Exp $
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -27,18 +27,18 @@
 #ifndef FUSE_SPEC48_H
 #define FUSE_SPEC48_H
 
-#ifndef FUSE_TYPES_H
-#include "types.h"
-#endif			/* #ifndef FUSE_TYPES_H */
+#include <libspectrum.h>
 
-BYTE spec48_readbyte(WORD address);
-BYTE spec48_read_screen_memory(WORD offset);
-void spec48_writebyte(WORD address, BYTE b);
+libspectrum_byte spec48_readbyte( libspectrum_word address );
+libspectrum_byte spec48_readbyte_internal( libspectrum_word address );
+libspectrum_byte spec48_read_screen_memory( libspectrum_word offset );
+void spec48_writebyte( libspectrum_word address, libspectrum_byte b );
+void spec48_writebyte_internal( libspectrum_word address, libspectrum_byte b );
 
-DWORD spec48_contend_memory( WORD address );
-DWORD spec48_contend_port( WORD port );
+libspectrum_dword spec48_contend_memory( libspectrum_word address );
+libspectrum_dword spec48_contend_port( libspectrum_word port );
 
-int spec48_init( machine_info *machine );
-int spec48_reset(void);
+int spec48_init( fuse_machine_info *machine );
+int spec48_reset( void );
 
 #endif			/* #ifndef FUSE_SPEC48_H */
