@@ -29,6 +29,7 @@
 #include "machine.h"
 #endif			/* #ifndef FUSE_MACHINE_H */
 
-int specplus2_init( machine_info *machine );
+int specplus2_init( fuse_machine_info *machine );
+int specplus2_reset( void );
 
 #endif			/* #ifndef FUSE_SPECPLUS2_H */
