@@ -1,7 +1,7 @@
 /* tape.h: tape handling routines
-   Copyright (c) 1999-2001 Philip Kendall
+   Copyright (c) 1999-2003 Philip Kendall
 
-   $Id: tape.h,v 1.11 2002/02/24 13:25:16 pak21 Exp $
+   $Id: tape.h,v 1.18 2003/07/11 11:03:25 pak21 Exp $
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -27,15 +27,19 @@
 #ifndef FUSE_TAPE_H
 #define FUSE_TAPE_H
 
-#ifndef FUSE_TYPES_H
-#include "types.h"
-#endif
+#include <libspectrum.h>
 
 int tape_init( void );
 
-int tape_open( const char *filename );
+int tape_open( const char *filename, int autoload );
+int tape_open_default_autoload( const char *filename );
+
+int tape_read_buffer( unsigned char *buffer, size_t length,
+		      libspectrum_id_t type, int autoload );
+
 int tape_close( void );
-int tape_rewind( void );
+int tape_select_block( size_t n );
+int tape_get_current_block( void );
 int tape_write( const char *filename );
 
 int tape_load_trap( void );
@@ -44,10 +48,13 @@ int tape_save_trap( void );
 int tape_play( void );
 int tape_toggle_play( void );
 
-int tape_next_edge( DWORD last_tstates );
+int tape_next_edge( libspectrum_dword last_tstates );
 
 int tape_stop( void );
 
+int tape_get_block_list( char ****list, size_t *n );
+int tape_free_block_list( char ***list, size_t n );
+
 extern int tape_playing;
 extern int tape_microphone;
 
