@@ -1,7 +1,7 @@
 /* timer.h: Speed routines for Fuse
    Copyright (c) 1999-2000 Philip Kendall
 
-   $Id: timer.h,v 1.6 2001/08/13 21:49:40 pak21 Exp $
+   $Id: timer.h,v 1.7 2002/10/03 21:32:06 pak21 Exp $
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -27,7 +27,7 @@
 #ifndef FUSE_TIMER_H
 #define FUSE_TIMER_H
 
-extern volatile int timer_count;
+extern volatile float timer_count;
 
 int timer_init(void);
 void timer_sleep(void);
