@@ -1,7 +1,7 @@
 /* gtkkeyboard.c: GTK+ routines for dealing with the keyboard
-   Copyright (c) 2000-2002 Philip Kendall, Russell Marks
+   Copyright (c) 2000-2003 Philip Kendall, Russell Marks
 
-   $Id: gtkkeyboard.c,v 1.10 2002/02/21 16:58:06 pak21 Exp $
+   $Id: gtkkeyboard.c,v 1.15 2003/06/20 20:22:19 pak21 Exp $
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -32,18 +32,18 @@
 #include <gdk/gdkx.h>
 #include <gtk/gtk.h>
 
+#include "compat.h"
 #include "gtkkeyboard.h"
 #include "gtkui.h"
-#include "keyboard.h"
 #include "keysyms.h"
-#include "widget/widget.h"
 
 static guint gtkkeyboard_unshift_keysym(guint keysym);
 
-int gtkkeyboard_keypress(GtkWidget *widget, GdkEvent *event,
-			 gpointer data)
+int
+gtkkeyboard_keypress( GtkWidget *widget GCC_UNUSED, GdkEvent *event,
+		      gpointer data GCC_UNUSED )
 {
-  guint keysym; keysyms_key_info *ptr;
+  guint keysym; const keysyms_key_info *ptr;
 
   keysym=gtkkeyboard_unshift_keysym(event->key.keyval);
 
@@ -51,14 +51,10 @@ int gtkkeyboard_keypress(GtkWidget *widget, GdkEvent *event,
 
   if(ptr) {
 
-    if( widget_level >= 0 ) {
-      widget_keyhandler( ptr->key1 );
-    } else {
-      if(ptr->key1 != KEYBOARD_NONE) keyboard_press(ptr->key1);
-      if(ptr->key2 != KEYBOARD_NONE) keyboard_press(ptr->key2);
-    }
+    if( ptr->key1 != KEYBOARD_NONE ) keyboard_press( ptr->key1 );
+    if( ptr->key2 != KEYBOARD_NONE ) keyboard_press( ptr->key2 );
+
     return TRUE;
-    
   }
 
   /* Now deal with the non-Speccy keys. Most are dealt with by
@@ -72,10 +68,11 @@ int gtkkeyboard_keypress(GtkWidget *widget, GdkEvent *event,
   return FALSE;
 }
 
-int gtkkeyboard_keyrelease(GtkWidget *widget, GdkEvent *event,
-			   gpointer data)
+int
+gtkkeyboard_keyrelease( GtkWidget *widget GCC_UNUSED, GdkEvent *event,
+		        gpointer data GCC_UNUSED )
 {
-  guint keysym; keysyms_key_info *ptr;
+  guint keysym; const keysyms_key_info *ptr;
 
   keysym=gtkkeyboard_unshift_keysym(event->key.keyval);
 
@@ -105,8 +102,9 @@ static guint gtkkeyboard_unshift_keysym(guint keysym)
 }
 
 /* Release all keys (called when focus lost) */
-int gtkkeyboard_release_all( GtkWidget *widget, GdkEvent *event,
-			     gpointer data )
+int
+gtkkeyboard_release_all( GtkWidget *widget GCC_UNUSED,
+			 GdkEvent *event GCC_UNUSED, gpointer data GCC_UNUSED )
 {
   keyboard_release_all();
   return TRUE;
