@@ -1,7 +1,7 @@
 /* gtkui.h: GTK+ routines for dealing with the user interface
-   Copyright (c) 2000-2002 Philip Kendall, Russell Marks
+   Copyright (c) 2000-2003 Philip Kendall, Russell Marks
 
-   $Id: gtkui.h,v 1.15 2002/02/26 01:22:37 pak21 Exp $
+   $Id: gtkui.h,v 1.21 2003/06/18 12:51:10 pak21 Exp $
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -39,6 +39,15 @@ extern GtkWidget* gtkui_window;
 extern GtkWidget* gtkui_drawing_area;
 
 void gtkui_destroy_widget_and_quit( GtkWidget *widget, gpointer data );
+char* gtkui_fileselector_get_filename( const char *title );
+
+void gtkui_load_binary_data( GtkWidget *widget, gpointer data );
+void gtkui_save_binary_data( GtkWidget *widget, gpointer data );
+int gtkui_confirm( const char *string );
+void gtk_tape_browse( GtkWidget *widget, gpointer data );
+void gtkui_roms( GtkWidget *widget, gpointer data );
+
+int gtkui_picture( const char *filename, int border );
 
 extern void gtkui_popup_menu(void);
 
