@@ -1,7 +1,7 @@
 /* xdisplay.h: Routines for dealing with the X display
    Copyright (c) 2000 Philip Kendall
 
-   $Id: xdisplay.h,v 1.12 2002/03/09 23:10:48 pak21 Exp $
+   $Id: xdisplay.h,v 1.13 2003/03/13 14:55:28 pak21 Exp $
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -27,6 +27,9 @@
 #ifndef FUSE_XDISPLAY_H
 #define FUSE_XDISPLAY_H
 
+int xdisplay_init( void );
+int xdisplay_end( void );
+
 int xdisplay_configure_notify(int width, int height);
 void xdisplay_area(int x, int y, int width, int height);
 
