@@ -1,7 +1,7 @@
-/* utils.c: some useful helper functions
-   Copyright (c) 1999-2002 Philip Kendall
+/* utils.h: some useful helper functions
+   Copyright (c) 1999-2003 Philip Kendall
 
-   $Id: utils.h,v 1.3 2002/01/15 21:33:50 pak21 Exp $
+   $Id: utils.h,v 1.9 2003/06/23 11:08:53 pak21 Exp $
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -27,13 +27,34 @@
 #ifndef FUSE_UTILS_H
 #define FUSE_UTILS_H
 
+#include <libspectrum.h>
+
+/* The ways we may have allocated memory for a file */
+typedef enum utils_file_open_mode {
+
+  UTILS_FILE_OPEN_MMAP,
+  UTILS_FILE_OPEN_MALLOC,
+
+} utils_file_open_mode;
+
+typedef struct utils_file {
+
+  unsigned char *buffer;
+  size_t length;
+
+  utils_file_open_mode mode;
+
+} utils_file;
+
+int utils_open_file( const char *filename, int autoload,
+		     libspectrum_id_t *type );
 int utils_find_lib( const char *filename );
-int utils_read_file( const char *filename, unsigned char **buffer,
-		     size_t *length );
-int utils_read_fd( int fd, const char *filename,
-		   unsigned char **buffer, size_t *length );
+int utils_read_file( const char *filename, utils_file *file );
+int utils_read_fd( int fd, const char *filename, utils_file *file );
+int utils_close_file( utils_file *file );
 
 int utils_write_file( const char *filename, const unsigned char *buffer,
 		      size_t length );
+const char* utils_get_temp_path( void );
 
 #endif			/* #ifndef FUSE_UTILS_H */
