@@ -1,7 +1,7 @@
 /* filesel.c: File selection dialog box
    Copyright (c) 2001,2002 Matan Ziv-Av, Philip Kendall, Russell Marks
 
-   $Id: filesel.c,v 1.16 2002/03/09 22:33:27 pak21 Exp $
+   $Id: filesel.c,v 1.27 2003/04/16 08:41:43 pak21 Exp $
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -26,6 +26,9 @@
 
 #include <config.h>
 
+#ifdef USE_WIDGET
+
+#include <sys/types.h>
 #include <dirent.h>
 #include <errno.h>
 #include <stdlib.h>
@@ -34,10 +37,9 @@
 #include <sys/stat.h>
 #include <unistd.h>
 
-#include "display.h"
+#include "fuse.h"
 #include "keyboard.h"
-#include "ui/uidisplay.h"
-#include "widget.h"
+#include "widget_internals.h"
 
 struct widget_dirent **widget_filenames; /* Filenames in the current
 					    directory */
@@ -63,6 +65,9 @@ static int widget_print_filename( struct widget_dirent *filename, int position,
 /* The filename to return */
 char* widget_filesel_name;
 
+/* Should we exit all widgets when we're done with this selector? */
+static int exit_all_widgets;
+
 static int widget_scandir( const char *dir, struct widget_dirent ***namelist,
 			   int (*select_fn)(const struct dirent*) )
 {
@@ -217,12 +222,21 @@ static int widget_scan_compare( const struct widget_dirent **a,
 
 /* File selection widget */
 
-int widget_filesel_draw( void* data )
+int
+widget_filesel_draw( void *data )
 {
   char *directory;
 
   int error;
 
+  /* Should we exit all widgets when we're done with the fileselector;
+     defaults to 'yes' if no data given for backward compatability reasons */
+  if( !data || *(int*)data ) {
+    exit_all_widgets = 1;
+  } else {
+    exit_all_widgets = 0;
+  }
+
   directory = widget_getcwd();
   if( directory == NULL ) return 1;
 
@@ -243,10 +257,9 @@ int widget_filesel_draw( void* data )
 
 int widget_filesel_finish( widget_finish_state finished ) {
 
-  /* Now return, either with a filename or without as appropriate */
-  if( finished == WIDGET_FINISHED_OK ) {
-    widget_filesel_name = widget_filenames[ current_file ]->name;
-  } else {
+  /* Return with null if we didn't finish cleanly */
+  if( finished != WIDGET_FINISHED_OK ) {
+    if( widget_filesel_name ) free( widget_filesel_name );
     widget_filesel_name = NULL;
   }
 
@@ -306,8 +319,7 @@ static int widget_print_all_filenames( struct widget_dirent **filenames, int n,
   }
 
   /* Display that lot */
-  uidisplay_lines( DISPLAY_BORDER_HEIGHT,
-		   DISPLAY_BORDER_HEIGHT + DISPLAY_HEIGHT );
+  widget_display_lines( 2, 20 );
 
   return 0;
 }
@@ -344,16 +356,23 @@ static int widget_print_filename( struct widget_dirent *filename, int position,
   return 0;
 }
 
-void widget_filesel_keyhandler( keyboard_key_name key )
+void
+widget_filesel_keyhandler( keyboard_key_name key, keyboard_key_name key2 )
 {
   char *fn, *ptr;
 
   new_current_file = current_file;
 
   switch(key) {
+  case KEYBOARD_Resize:		/* Fake keypress used on window resize */
+    widget_dialog_with_border( 1, 2, 30, 20 );
+    widget_print_all_filenames( widget_filenames, widget_numfiles,
+				top_left_file, current_file        );
+    break;
+    
   case KEYBOARD_1:		/* 1 used as `Escape' generates `EDIT',
 				   which is Caps + 1 */
-    widget_return[ widget_level ].finished = WIDGET_FINISHED_CANCEL;
+    if( key2 == KEYBOARD_Caps ) widget_end_widget( WIDGET_FINISHED_CANCEL );
     break;
   
   case KEYBOARD_5:		/* Left */
@@ -400,7 +419,7 @@ void widget_filesel_keyhandler( keyboard_key_name key )
     /* Get the new directory name */
     fn = widget_getcwd();
     if( fn == NULL ) {
-      widget_return[ widget_level ].finished = WIDGET_FINISHED_CANCEL;
+      widget_end_widget( WIDGET_FINISHED_CANCEL );
       return;
     }
     ptr = fn;
@@ -410,15 +429,19 @@ void widget_filesel_keyhandler( keyboard_key_name key )
     );
     if( fn == NULL ) {
       free( ptr );
-      widget_return[ widget_level ].finished = WIDGET_FINISHED_CANCEL;
+      widget_end_widget( WIDGET_FINISHED_CANCEL );
       return;
     }
     strcat( fn, "/" ); strcat( fn, widget_filenames[ current_file ]->name );
 			
     if(chdir(fn)==-1) {
       if(errno==ENOTDIR) {
-	free( fn );
-	widget_end_all( WIDGET_FINISHED_OK );
+	widget_filesel_name = fn;
+	if( exit_all_widgets ) {
+	  widget_end_all( WIDGET_FINISHED_OK );
+	} else {
+	  widget_end_widget( WIDGET_FINISHED_OK );
+	}
       }
     } else {
       widget_scan( fn ); free( fn );
@@ -462,9 +485,7 @@ void widget_filesel_keyhandler( keyboard_key_name key )
       widget_print_filename( widget_filenames[ new_current_file ],
 			     new_current_file - top_left_file, 1 );
         
-      uidisplay_lines(DISPLAY_BORDER_HEIGHT,
-		      DISPLAY_BORDER_HEIGHT + DISPLAY_SCREEN_HEIGHT );
-	  
+      widget_display_lines( 2, 20 );
     }
 
     /* Reset the current file marker */
@@ -473,3 +494,5 @@ void widget_filesel_keyhandler( keyboard_key_name key )
   }
 
 }
+
+#endif				/* #ifdef USE_WIDGET */
